import numpy as np
import numpy.typing as npt


# TODO: add reverse from structured to unstructured
def cast_array(
    in_array: np.ndarray,
    cast_type: npt.DTypeLike,
) -> np.ndarray:
    """Returns a copy of the passed array in the desired type.
    """
    cast_type = np.dtype(cast_type)
    if in_array.dtype == cast_type:
        cast_array = in_array
    else:
        multi_dim = in_array.ndim > 1
        input_col_num = in_array.shape[-1]
        if cast_type.fields is None:
            target_col_num = 1
        else:
            target_col_num = len(cast_type.fields)
        if multi_dim and input_col_num != target_col_num:
            raise ValueError(
                    f'Casting to {cast_type} requires in_array with '
                    + f'{target_col_num} columns, but you passed an '
                    + f'array with shape {in_array.shape}.'
                    )
        cast_array = in_array.astype(cast_type.descr[0][1])
        cast_array = cast_array.copy()
        cast_array = cast_array.view(dtype=cast_type, type=np.ndarray)
        cast_array = cast_array.squeeze()
    return cast_array



from pathlib import Path

import yaml


THIS_DIR = Path(__file__).parent

def parse_yaml(path: str):
    with open(path, 'r') as f:
        data = yaml.load(f, Loader=yaml.FullLoader)
    return data

def struc_type(type_dict: dict) -> list:
    return list(type_dict.items())

_immutable_struc = lambda x: tuple(struc_type(x))

class Types:
    def __init__(self, config_file: str=str(THIS_DIR / 'types.yml')):
        data = parse_yaml(config_file)
        for key, val in data['TYPE'].items():
            setattr(self, key, val)
        for key, val in data['PHYSICS_TYPE'].items():
            setattr(self, key, val)
        self.__pmu = _immutable_struc(data['PMU_DTYPE'])
        self.__edge = _immutable_struc(data['EDGE_DTYPE'])
        self.__color = _immutable_struc(data['COLOR_DTYPE'])

    @property
    def pmu(self):
        return list(self.__pmu)

    @property
    def edge(self):
        return list(self.__edge)

    @property
    def color(self):
        return list(self.__color)
